<?php

namespace GunTab\Api\GunTab;

use GuzzleHttp\Client as Guzzle;
use GunTab\Api\GunTab\RequestObjects\Invoice;
use Dragon\Api\GuzzleTrait;

class Client {
	use GuzzleTrait;
	
	const API_URL = 'https://api.guntab.com/v1/';
	
	public function __construct(string $apiKey) {
		$this->guzzle = new Guzzle([
			'base_uri' => static::API_URL,
			'headers' => [
				'Authorization' => 'Token ' . $apiKey,
			],
		]);
	}
	
	public function createInvoice(Invoice $invoice) : ?\stdClass {
		$response = $this->jsonCall(function () use ($invoice) {
			$listings = [];
			
			foreach ($invoice->listings as $listing) {
				$listings[] = [
					'title'			=> $listing->title,
					'amount_cents'	=> $listing->amount_cents,
					'quantity'		=> $listing->quantity,
					'url'			=> $listing->url,
				];
			}
			
			return $this->guzzle->post('invoices', [
				'json' => [
					'shipping_amount_cents' => $invoice->shipping_amount_cents,
					'sales_tax_amount_cents' => $invoice->sales_tax_amount_cents,
					'amount_cents'	=> $invoice->amount_cents,
					'listings' => $listings,
				],
			]);
		});
		
		if (!empty($this->getError())) {
			return null;
		}
		
		return $response;
	}
}
