<?PHP

use Dragon\Form\DropDown;
use Dragon\Config;
use Dragon\Form\InputBox;
use Dragon\Form\TextArea;

if (array_key_exists('notice', $pageData)) {
	echo $pageData['notice'];
}

$prefix = Config::$namespace;
?>

<div class="card card-full-width">
	<div class="<?= $prefix; ?>-intro">	
		<h1><?= get_admin_page_title() ?></h1>			
	</div>

	<form method="POST">
		<input name="<?= $prefix; ?>_nonce" value="<?php echo wp_create_nonce($prefix . '_nonce') ?>" type="hidden" />
		<table class="form-table">
			<tr>
				<td>
					<input name="<?= $prefix; ?>_save_settings" type="submit" value="Save All" class="button button-primary" />
				</td>
			</tr>
			<?php
			foreach ($pageData['sections'] as $sectionName => $fields) {
				?>
				<tr>
					<td colspan="2">
						<h2><?= $sectionName; ?></h2>
					</td>
				</tr>
				<?php
				
				foreach ($fields as $fieldName => $fieldData) {
					$value = get_option($fieldName, old($fieldName));
					if (!empty($fieldData['encrypted'])) {
						$value = dragonDecrypt($value);
					}
					?>
					<tr class="form-field <?= !empty($fieldData['required']) ? 'form-required' : '' ?>">
						<th scope="row">
							<label for="<?= $fieldName; ?>">
								<?php
								echo $fieldData['label'];
								if (!empty($fieldData['required'])) {
									echo ' <span class="description">(required)</span>';
								}
								?>
							</label>
						</th>
						<td>
							<?php
							if (empty($fieldData['attributes'])) {
								$fieldData['attributes'] = [];
							}
							
							$fieldData['attributes'] = array_merge($fieldData['attributes'], [
								'id'	=> $fieldName,
								'name'	=> $fieldName,
							]);
							
							switch($fieldData['type']) {
								case 'text':
								case 'number':
								case 'date':
								case 'email':
								case 'url':
									$fieldData['attributes']['value'] = $value;
									echo InputBox::create($fieldData);
									break;
								case 'textarea':
									echo TextArea::create($fieldData, $value);
									break;
								case 'checkbox':
									echo InputBox::create($fieldData);
									break;
								case 'select':
									echo DropDown::create($fieldData, $value);
									break;
							}
							
							if (!empty($fieldData['description'])) {
								echo '<p class="description">' . $fieldData['description'] . '</p>';
							}
							?>
						</td>
					</tr>
					<?php
				}
			}
			?>
			<tr>
				<td>
					<input name="<?= $prefix; ?>_save_settings" type="submit" value="Save All" class="button button-primary" />
				</td>
			</tr>
		</table>
	</form>
</div>