<?PHP

use Dragon\Form\DropDown;
use Dragon\Form\InputBox;
use Dragon\Form\TextArea;
use Dragon\Config;
use Dragon\Form\EmailMessage;

if (array_key_exists('notice', $pageData)) {
	echo $pageData['notice'];
}

$prefix = Config::$namespace;

?>
<div class="card card-full-width">
	<div class="<?= $prefix; ?>-intro">	
		<h1><?= get_admin_page_title(); ?></h1>
	</div>

	<form method="POST">
		<input name="<?= $prefix; ?>_nonce" value="<?php echo wp_create_nonce($prefix . '_nonce') ?>" type="hidden" />
		<table class="form-table">
			<?php
			foreach ($pageData['fields'] as $dbColumn => $fieldData) {
				$fieldName = 'details_' . $dbColumn;
				$value = old($fieldName) ?? $pageData['model'][$dbColumn] ?? '';
				?>
				<tr class="form-field <?= !empty($fieldData['required']) ? 'form-required' : '' ?>">
					<th scope="row">
						<label for="<?= $fieldName; ?>">
							<?php
							echo $fieldData['label'];
							if (!empty($fieldData['required'])) {
								echo '<span class="description">(required)</span>';
							}
							?>
						</label>
					</th>
					<td>
						<?php
						if (empty($fieldData['attributes'])) {
							$fieldData['attributes'] = [];
						}
						
						$fieldData['attributes'] = array_merge($fieldData['attributes'], [
							'id'	=> $fieldName,
							'name'	=> $fieldName,
						]);
						
						switch($fieldData['type']) {
							case 'text':
							case 'number':
							case 'date':
							case 'email':
							case 'url':
								$fieldData['attributes']['value'] = $value;
								echo InputBox::create($fieldData);
								break;
							case 'checkbox':
								echo InputBox::create($fieldData);
								break;
							case 'textarea':
								echo TextArea::create($fieldData, $value);
								break;
							case 'select':
								echo DropDown::create($fieldData, $value);
								break;
							case 'email-message':
								echo EmailMessage::create($fieldData);
								break;
						}
						
						if (!empty($fieldData['description'])) {
							echo '<p class="description">' . $fieldData['description'] . '</p>';
						}
						?>
					</td>
				</tr>
				<?php
			}
			?>
			<tr>
				<td>
					<input name="<?= $prefix; ?>_save" type="submit" value="Save" class="button button-primary" />
				</td>
			</tr>
		</table>
	</form>
</div>