<?php

namespace Dragon\Abstracts;

use Dragon\Url;
use Dragon\View;
use Illuminate\Database\Eloquent\Model;
use Dragon\Input;

abstract class DetailsAbstract extends OptionsAbstract
{
	protected $pageData = [];
	protected string $listPageSlug = "";
	protected string $modelName = "";
	
	protected array $dbColumnToField = [];
	protected array $dbColumnToStaticValue = [];
	
	public function render()
	{
		$this->input = new Input();
		$this->setDbColumnToField();
		$this->setRequiredFields();
		$this->saveSettings($this->pageData);
		
		$this->pageData['model'] = $this->maybeGetModel() ?? [];
		$this->pageData['fields'] = $this->dbColumnToField;
		
		View::displayPage('Table/Details', $this->pageData);
	}
	
	protected function save()
	{
		$this->throwIfInvalid();
		$this->saveFields();
	}
	
	protected function throwIfInvalid()
	{
		//
	}
	
	protected function getId($key)
	{
		return $this->input->hasGet($key) && is_numeric($this->input->get($key)) ? (int)$this->input->get($key) : null;
	}
	
	protected function redirectToList()
	{
		$link = Url::getAdminMenuLink($this->listPageSlug);
		wp_redirect($link);
		exit;
	}
	
	protected function maybeGetModel()
	{
		if (is_numeric($this->input->get('id'))) {
			return $this->getModel((int)$this->input->get('id'));
		}
		
		return null;
	}
	
	protected function saveFields()
	{
		$model = $this->maybeGetModel();
		if (empty($model)) {
			$model = new $this->modelName();
		}
		
		foreach ($this->dbColumnToField as $dbColumn => $fieldData) {
			if (!empty($fieldData['fake_column']) && $fieldData['fake_column'] === true) {
				continue;
			}
			$model->{$dbColumn}	= $this->input->post('details_' . $dbColumn);
		}
		
		foreach ($this->dbColumnToStaticValue as $dbColumn => $value) {
			$model->{$dbColumn}	= $value;
		}
		
		$this->handleAdditionalModelData($model);
		$model->save();
		
		$this->finishingTouches($model);
		$this->redirectToList();
	}
	
	protected function getModel($id)
	{
		return call_user_func([$this->modelName, 'find'], $id);
	}
	
	protected function setDbColumnToField() {
		//
	}
	
	protected function handleAdditionalModelData(Model &$model) {
		//
	}
	
	protected function finishingTouches(Model $model) {
		//
	}
	
	protected function getMissingFields() {
		$mapped = [];
		foreach ($this->missingFields as $fieldName) {
			$dbColumn = substr($fieldName, strlen('details_'));
			$mapped[$fieldName] = $this->dbColumnToField[$dbColumn]['label'];
		}
		
		return $mapped;
	}
	
	private function setRequiredFields() {
		foreach ($this->dbColumnToField as $dbColumn => $fieldData) {
			if (!empty($fieldData['required'])) {
				$this->requiredFields[] = 'details_' . $dbColumn;
			} else {
				$this->optionalFields[] = 'details_' . $dbColumn;
			}
		}
	}
}
