<?php

namespace Dragon\Abstracts;

use Illuminate\Pagination\Paginator;
use Dragon\Url;
use Dragon\Input;
use Dragon\View;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Builder;
use Dragon\Csv;
use Illuminate\Support\Str;

abstract class TableAbstract extends PageAbstract
{
	protected $headers = [
		'Title',
		'Comments',
		'Author',
	];
	
	protected $viewPageSlug = "";
	protected $createPageSlug = "";
	protected $updatePageSlug = "";
	protected $currentPageSlug = "";
	
	protected $viewLink = "";
	protected $createLink = "";
	protected $updateLink = "";
	protected $currentPageLink = "";
	
	protected $customLinks = [
		//
	];
	
	protected $actionButtons = [
		//
	];
	
	protected $pageText = "";
	
	protected $canView = false;
	protected $canCreate = false;
	protected $canUpdate = false;
	protected $canDelete = false;
	
	protected $searchable = false;
	
	protected $searchableByDate = false;
	protected $dateSearchColumnName = Model::CREATED_AT;
	
	protected $filters = [
		//
	];
	
	protected $rows = [
// 		[
// 			[
// 				'data'	=> 'Testing',
// 				'links'	=> [
// 					'view'	=> [
// 						'link'	=> '#',
// 					],
// 					'edit'	=> [
// 						'link'	=> '#',
// 						'class'	=> 'my-edit',
// 					],
// 					'delete'	=> [
// 						'link'	=> '#',
// 					],
// 				],
// 			],
// 			[
// 				'data'	=> 'Testing 1',
// 			],
// 			[
// 				'data'	=> 'Testing 2',
// 			],
// 		],
	];
	
	protected $pageData = [];
	protected $paging = [
		'per_page'	=> 20,
	];
	
	protected $paginationQueryVar = 'paged';
	protected $input = null;
	
	protected $csvHeaders = [];
	
	public function setPaginator()
	{
		Paginator::currentPageResolver(function() {
			return (int)$this->input->get($this->paginationQueryVar, 1);
		});
	}
	
	public function render()
	{
		$this->input = new Input();
		$this->setPaginator();
		
		if (empty($this->createPageSlug)) {
			$this->createPageSlug = $this->currentPageSlug . '-details';
		}
		
		if (empty($this->updatePageSlug)) {
			$this->updatePageSlug = $this->currentPageSlug . '-details';
		}
		
		$this->createLink = $this->canCreate ? Url::getAdminMenuLink($this->createPageSlug) : '';
		$this->updateLink = $this->canUpdate ? Url::getAdminMenuLink($this->updatePageSlug) : '';
		$this->viewLink = $this->canView ? Url::getAdminMenuLink($this->viewPageSlug) : '';
		$this->currentPageLink = Url::getAdminMenuLink($this->currentPageSlug);
		$this->maybeDeleteListItem();
		$this->display();
		
		$this->pageData = array_merge([
			'create-link'	=> $this->createLink,
			'pageText'		=> $this->pageText,
			'headers'		=> $this->headers,
			'rows'			=> $this->rows,
			'paging'		=> $this->paging,
			'filters'		=> $this->filters,
			'searchable'	=> $this->searchable,
			'dateSeachable'	=> $this->searchableByDate,
			'actionButtons'	=> $this->actionButtons,
		], $this->pageData);
		
		View::displayPage('Table/AdminTable', $this->pageData);
	}
	
	protected function display()
	{
		$this->populateRows();
	}
	
	protected function populateRows()
	{
		$search = $this->input->get('s');
		$listLink = Url::getAdminMenuLink($this->currentPageSlug);
		$list = $this->getModelQuery();
		
		if ($this->searchableByDate) {
			$list = $this->maybeAddDateSearch($list);
		}
		
		$list = $list->paginate($this->paging['per_page'])
		->setPageName($this->paginationQueryVar)
		->withPath($listLink);
		
		if (!empty($search)) {
			$list = $list->appends(['s' => $search]);
		}
		
		$pagingData = $list->toArray();
		unset($pagingData['data']);
		$this->paging = $pagingData;
		
		foreach ($list as $listItem) {
			
			if ($this->addRow($listItem) === false) {
				continue;
			}
			
			$idKey = $listItem->getKeyName();
			$this->maybeAddRowLinks($listItem->{$idKey}, $listItem);
			
		}
	}
	
	protected function getModelQuery()
	{
		//
	}
	
	protected function addRow($listItem)
	{
		//
	}
	
	protected function maybeDeleteListItem()
	{
		if ($this->canDelete && is_numeric($this->input->get('delete'))) {
			
			$attorney = $this->getModelToDelete((int)$this->input->get('delete'));
			
			if (!empty($attorney)) {
				$attorney->delete();
			}
			
			$this->redirectToList();
			
		}
	}
	
	protected function getModelToDelete($id)
	{
		return null;
	}
	
	protected function redirectToList()
	{
		wp_redirect($this->currentPageLink);
		exit;
	}
	
	protected function maybeAddRowLinks($modelId, $model)
	{
		$rowId = count($this->rows)-1;
		$this->rows[$rowId][0]['links'] = [];
		
		if ($this->canView) {
			
			$this->rows[$rowId][0]['links']['view'] = [
				'link'	=> $this->getViewLink($modelId),
			];
			
		}
		
		if ($this->canUpdate) {
			
			$this->rows[$rowId][0]['links']['edit'] = [
				'link'	=> $this->getUpdateLink($modelId),
			];
			
		}
		
		if ($this->canDelete) {
			
			$this->rows[$rowId][0]['links']['delete'] = [
				'link'	=> $this->getDeleteLink($modelId),
			];
			
		}
		
		foreach ($this->customLinks as $type => $data) {
			
			if ($this->{'rowHas' . kababToCamel($type)}($model)) {
				
				$this->rows[$rowId][0]['links'][$type] = [
					'link'	=> $this->{$data['method']}($model),
					'name'	=> $data['name'],
				];
				
			}
			
		}
	}
	
	protected function getDeleteLink($id)
	{
		return $this->currentPageLink .
			'&delete=' . $id .
			'&paged=' . $this->paging['current_page'] .
			'&s=' . $this->input->get('s');
	}
	
	protected function getUpdateLink($id)
	{
		return $this->updateLink . '&id=' . $id;
	}
	
	protected function getViewLink($id)
	{
		return $this->viewLink . '&id=' . $id;
	}
	
	protected function maybeAddDateSearch(Builder $query) {
		$startDate = $this->input->get('date-start');
		$endDate = $this->input->get('date-end');
		
		if (!empty($startDate)) {
			$query = $query->whereDate($this->dateSearchColumnName, '>=', $startDate);
		}
		
		if (!empty($endDate)) {
			$query = $query->whereDate($this->dateSearchColumnName, '<=', $endDate);
		}
		
		return $query;
	}
	
	protected function maybeDownloadToCsv(string $queryParam, string $filename) {
		$shouldDownload = $this->input->get($queryParam) !== null;
		
		if ($shouldDownload === false) {
			return;
		}
		
		$data = $this->getModelQuery()->get();
		$csv = new Csv();
		$csv->headers = $this->csvHeaders;
		
		foreach ($data as $row) {
			$rowData = $this->{Str::camel($queryParam) . 'AddRow'}($row);
			if (!empty($rowData)) {
				$csv->rows[] = $rowData;
			}
		}
		
		$csv->download($filename);
	}
}
