<?php

namespace Dragon\Hooking;

use Dragon\Config;
use Dragon\Input;
use Dragon\Email\EmailMessageParser;
use Dragon\Models\EmailMessage;

class Ajax
{
	protected static ?Input $input = null;
	
	public static function init()
	{
		static::$input = new Input();
	}
	
	public static function getEmailTemplate() {
		$emailTemplate = static::$input->post('email_template');
		$emailId = static::$input->post('email_id');
		$data = null;
		if (!empty($emailId)) {
			$emailMessage = EmailMessage::find($emailId);
			$data = $emailMessage->message;
		}
		
		return [
			'template' => (string)EmailMessageParser::makeTemplateFor($emailTemplate, $data),
		];
	}
	
	protected static function createHook(
		$hook,
		Callable $callback,
		$includeNoPrivledges = false,
		bool $dragonNs = false
	) {
		$hookName = $dragonNs ? 'dragon_' . $hook : Config::$namespace . '_' . $hook;
		
		add_action('wp_ajax_' . $hookName, function () use ($callback) {
			static::doCallback($callback);
		});
		
		if ($includeNoPrivledges) {
			add_action('wp_ajax_nopriv_' . $hookName, function () use ($callback) {
				static::doCallback($callback);
			});
		}
	}
	
	private static function doCallback(Callable $callback)
	{
		$response = $callback();
		if (is_array($response) || is_object($response)) {
			echo json_encode($response);
		} else {
			echo '{}';
		}
		
		wp_die();
	}
}
