<?php

namespace Dragon;

class User
{
	const USER_KEY_BASE = 'dragon.user_data';
	const ROLE_ADMIN = 'administrator';
	
	public static function isLoggedIn()
	{
		$user = wp_get_current_user();
		return $user->exists();
	}
	
	public function logout() {
		wp_logout();
		wp_redirect(wp_login_url());
	}
	
	public static function getUserId()
	{
		return get_current_user_id();
	}
	
	public static function isAdmin($userId = null) {
		$roles = static::getRoles($userId);
		return is_array($roles) && in_array(static::ROLE_ADMIN, $roles);
	}
	
	public static function getRole($userId = null)
	{
		$roles = static::getRoles($userId);
		return is_array($roles) && !empty($roles[0]) ? $roles[0] : false;
	}
	
	public static function getRoles($userId = null)
	{
		$user = $userId !== null ? new \WP_User( $userId ) : wp_get_current_user();
		return !empty($user->roles) ? $user->roles : false;
	}
	
	public static function addRole($userId, string $role){
		$user = new \WP_User($userId);
		$user->set_role($role);
	}
	
	public static function getMeta($key = null, $default = null, $wasPosted = false, $userId = null)
	{
		$userId = empty($userId) ? static::getUserId() : $userId;
		$data = get_user_meta($userId, $key);
		if (empty($key)) {
			return get_user_meta($userId, $key);
		}
		
		if (is_array($data)) {
			
			if (count($data) === 1 && array_key_exists(0, $data)) {
				$data = $data[0];
			}
			
			if (empty($data)) {
				
				$input = new Input();
				return $wasPosted && !empty($input->post($key)) ? sanitize_text_field($input->post($key)) : $default;
			}
			
		}
		
		return $data;
	}
	
	public static function getAllMeta($userId)
	{
		return static::getMeta(null, null, false, $userId);
	}
	
	public static function setMeta($key, $value, $userId = null)
	{
		$userId = empty($userId) ? static::getUserId() : $userId;
		update_user_meta($userId, $key, $value);
	}
	
	public static function deleteMeta($key, $userId = null)
	{
		$userId = empty($userId) ? static::getUserId() : $userId;
		delete_user_meta($userId, $key);
	}
	
	public static function getUserIds(array $args = [])
	{
		$ids = [];
		$users = get_users($args);
		foreach ($users as $user) {
			$ids[] = $user->ID;
		}
		
		return $ids;
	}
	
	public static function redirectIfNotLoggedIn()
	{
		if (static::isLoggedIn() === false) {
			auth_redirect();
		}
	}
	
	public static function getUserOption($key, $default = null, $forcedUserId = false)
	{
		$key = static::getKeyNameForUser($key, $forcedUserId);
		return get_option($key, $default);
	}
	
	public static function setUserOption($key, $data, $forcedUserId = false)
	{
		$key = static::getKeyNameForUser($key, $forcedUserId);
		update_option($key, $data);
	}
	
	public static function loginAs($userId)
	{
		wp_set_auth_cookie($userId);
	}
	
	public static function register($username, $password, $email)
	{
		wp_create_user($username, $password, $email);
	}
	
	private static function getKeyNameForUser($key, $forcedUserId = false)
	{
		$userId = $forcedUserId !== false ? (int)$forcedUserId : static::getUserId();
		
		$keyName = static::USER_KEY_BASE . '.' . $userId . '.' . $key;
		return esc_sql($keyName);
	}
}
